package generator.xml;


import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.w3c.dom.Element;

import gr.uom.java.xmi.UMLAssociation;
import gr.uom.java.xmi.UMLAssociationEnd;
import gr.uom.java.xmi.UMLModel;

public class XMLAssociationGenerator extends XMLGenerator {
	
	private List<UMLAssociation> associationList;
	Map<UMLAssociationEnd, List<UMLAssociationEnd>> associationMap =
		new HashMap<UMLAssociationEnd, List<UMLAssociationEnd>>();
	Set<UMLAssociationEnd> parents = new HashSet<UMLAssociationEnd>();

	public XMLAssociationGenerator(UMLModel aModel, String element) {
		super(aModel, element);
		
		associationList = aModel.getAssociationList();
		initializeAssociationMap();

		Element root = document.createElement("VirtualRoot");
		root.setAttribute("name", "VirtualRoot");
		for(int i = 0; i < associationList.size(); i++){
			if(!parents.contains(associationList.get(i).getEnd1())){
				Element assocElement = associates(associationList.get(i).getEnd1(), false);
				if (assocElement != null)
					root.appendChild(assocElement);
			}
		}
		document.appendChild(root);	
	}
	
	private Element associates(UMLAssociationEnd end1, boolean elementFlag){
		Element assocElement = document.createElement("Class");
		//parents.add(end1);
		if (end1.getName() != null)
			assocElement.setAttribute("_name", end1.getName());
		assocElement.setAttribute("ID", Long.toString(end1.getID()));
		assocElement.setAttribute("participant", end1.getParticipant());
		assocElement.setAttribute("aggregation", end1.getAggregation());
		//assocElement.setAttribute("multiplicityLower", Integer.toString(end1.getMultiplicityLower()));
		//assocElement.setAttribute("multiplicityUpper", Integer.toString(end1.getMultiplicityUpper()));
		
		if (end1.getParticipant() != null && end1.getParticipant().equals(element)){ 
			elementFlag = true;
			parents.add(end1);
		}
		if(!associationMap.containsKey(end1))
			return assocElement;
		
		List<UMLAssociationEnd> ends2List = associationMap.get(end1);
		for(int i = 0; i < ends2List.size(); i++){
			if (element == null)
				elementFlag = true;
			else if (ends2List.get(i).getParticipant() != null && ends2List.get(i).getParticipant().equals(element))
				elementFlag = true;
			assocElement.appendChild(associates(ends2List.get(i), elementFlag));	
		}
		
		if (elementFlag)
			return assocElement;

		return null;
	}
	
	private void initializeAssociationMap(){
		for(int i = 0; i < associationList.size(); i++){
			if(associationMap.containsKey(associationList.get(i).getEnd1())){
				List<UMLAssociationEnd> children = associationMap.get(associationList.get(i).getEnd2());
				children.add(associationList.get(i).getEnd2());
				associationMap.put(associationList.get(i).getEnd1(), children);
			}
			else{
				List<UMLAssociationEnd> children = new ArrayList<UMLAssociationEnd>();
				children.add(associationList.get(i).getEnd2());
				associationMap.put(associationList.get(i).getEnd1(), children);
			}
		}
	}

}
