package generator.xml;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import org.w3c.dom.Element;

import gr.uom.java.xmi.UMLCallAction;
import gr.uom.java.xmi.UMLClass;
import gr.uom.java.xmi.UMLClassifierRole;
import gr.uom.java.xmi.UMLCollaboration;
import gr.uom.java.xmi.UMLMessage;
import gr.uom.java.xmi.UMLModel;
import gr.uom.java.xmi.UMLOperation;
import gr.uom.java.xmi.UMLParameter;

public class XMLCollaborationGenerator extends XMLGenerator{

	private Map<UMLMessage, List<UMLMessage>> messagesMap = new HashMap<UMLMessage, List<UMLMessage>>();
	private Element classElement;
	
	public XMLCollaborationGenerator(UMLModel aModel, String element) {
		super(aModel, element);
		
		UMLClass umlClass = model.getClass(element);
		createMessagesMap(element);
		Element root = document.createElement("VirtualRoot");
		root.setAttribute("name", "VirtualRoot");
		classElement = document.createElement("Class");
		classElement.setAttribute("name", umlClass.getName());
		root.appendChild(classElement);
		addCollaborationElements();
		document.appendChild(root);	
	}
	
	private void addCollaborationElements(){
		for(Entry<UMLMessage, List<UMLMessage>> entry : messagesMap.entrySet()){
			if (entry.getKey().getAction() instanceof UMLCallAction && ((UMLCallAction) entry.getKey().getAction()).getOperation() != null){
				Element operationElement = document.createElement("Operation");
				UMLOperation operation = ((UMLCallAction) entry.getKey().getAction()).getOperation();
				operationElement.setAttribute("ID", Long.toString(operation.getID()));
				operationElement.setAttribute("operationName", operation.getName());
				operationElement.setAttribute("visibility", operation.getVisibility());
				
				List<UMLParameter> parameters = operation.getParameters();
				Collections.sort(parameters, new NameComparator());
				
				if(parameters != null){
					int k = 0;
					for (UMLParameter parameter : parameters){
						String counter = null;	
						if(k < 10)
							counter = "0" + k;
						else
							counter = String.valueOf(k);
							operationElement.setAttribute("param" + counter, parameter.getType());
							k++;
							Element paramElement = document.createElement("Parameter");
							if(!parameter.getKind().equals("return"))
								paramElement.setAttribute("paramName", parameter.getName());
							paramElement.setAttribute("paramKind", parameter.getKind());
							paramElement.setAttribute("paramtType", parameter.getType());

							operationElement.appendChild(paramElement);
						}
					}
				
				List<UMLMessage> messages = entry.getValue();
				for(UMLMessage message : messages){
					if (message.getAction() instanceof UMLCallAction){
						UMLOperation operationCall = ((UMLCallAction) message.getAction()).getOperation();
						Element operationCallElement = document.createElement("MethodCall");
						operationCallElement.setAttribute("ID", Long.toString(operationCall.getID()));
						operationCallElement.setAttribute("name", operationCall.getName());
						operationCallElement.setAttribute("visibility", operationCall.getVisibility());
						
						List<UMLParameter> ocParams = operation.getParameters();
						Collections.sort(ocParams, new NameComparator());
						int k = 0;
						for (UMLParameter parameter : parameters){
							operationCallElement.setAttribute("arg" + (k+1), parameter.getType());
							Element paramElement = document.createElement("Argument");
							paramElement.setAttribute("argType", parameter.getType());
							paramElement.setAttribute("argKind", parameter.getType());
							operationCallElement.appendChild(paramElement);
						}
						operationElement.appendChild(operationCallElement);
					}
				}
				classElement.appendChild(operationElement);
			}
		}
	}
	
	private void createMessagesMap(String element){
		List<UMLCollaboration> collaborationList = collaborationList = model.getCollaborationList();

		//sender = customer, not reciever
		for(UMLCollaboration collaboration : collaborationList){
			List<UMLMessage> messages = collaboration.getMessageList();
			for(UMLMessage message : messages){
				UMLMessage predecessor = message.getPredecessorMessage();
				if (predecessor != null){
					UMLClassifierRole sender = predecessor.getSender();
					if (sender.getBase() != null && sender.getBase().equals(element)){
						if (!messagesMap.containsKey(predecessor)) messagesMap.put(predecessor, new ArrayList<UMLMessage>()); 
						List<UMLMessage> calls = messagesMap.get(predecessor);
						calls.add(message);
					}
				}
			}
		}
		
	}
}
