package generator.xml;

import java.util.Collections;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.Stack;

import org.w3c.dom.Element;

import gr.uom.java.xmi.UMLAttribute;
import gr.uom.java.xmi.UMLClass;
import gr.uom.java.xmi.UMLModel;
import gr.uom.java.xmi.UMLOperation;
import gr.uom.java.xmi.UMLParameter;

public class XMLContainmentGenerator extends XMLGenerator {
	Stack<String> classSet = new Stack<String>();
	Set<String> usedClasses = new HashSet<String>();
	boolean parent;
	public XMLContainmentGenerator(UMLModel aModel, String element) {
		super(aModel, element);

		classes = model.getClassList();
		Element root = document.createElement("VirtualRoot");
		root.setAttribute("name", "VirtualRoot");
		if (element != null){
			for(int i = 0; i < classes.size(); i++){
				if (classes.get(i).getName().equals(element)){
					root.appendChild(contains((UMLClass)classes.get(i)));
				}
			}
		}
		else 
			for(int i = 0; i < classes.size(); i++){
				root.appendChild(contains((UMLClass)classes.get(i)));
			}
		document.appendChild(root);	
		
	}
	
	private Element contains(UMLClass umlClass){
		Element classElement = document.createElement("Class");
		classElement.setAttribute("ID", Long.toString(umlClass.getID()));
		classElement.setAttribute("className", umlClass.getName());
		classElement.setAttribute("isAbstract", Boolean.toString(umlClass.isAbstract()));
		classElement.setAttribute("isInterface", Boolean.toString(umlClass.isInterface()));
		
		List<UMLOperation> operations = umlClass.getOperations();
		Collections.sort(operations, new NameComparator());
		for(UMLOperation operation : operations){
			Element operationElement = document.createElement("Operation");
			operationElement.setAttribute("operationName", operation.getName());
			operationElement.setAttribute("visibility", operation.getVisibility());
			operationElement.setAttribute("ID", Long.toString(operation.getID()));
			
						
			List<UMLParameter> parameters = operation.getParameters();
			Collections.sort(parameters, new NameComparator());
			
			if(parameters != null){
				int k = 0;
				UMLParameter returnParameter = null;
				for(UMLParameter parameter : parameters){
					if (!parameter.getKind().equals("return"))
						operationElement.setAttribute("param" + (++k), getTypeFromFullQualifiedName(parameter.getType()));
					else returnParameter = parameter;
				}
				if (returnParameter != null)
					operationElement.setAttribute("param" + (++k), getTypeFromFullQualifiedName(returnParameter.getType()));
			}
			
			if(parameters != null){
				for (UMLParameter parameter : parameters){
					if(!parameter.getKind().equals("return")){
						Element paramElement = document.createElement("Parameter");
						paramElement.setAttribute("paramName", parameter.getName());
						paramElement.setAttribute("paramKind", parameter.getKind());
						//paramElement.setAttribute("paramType", parameter.getType());
						paramElement.setAttribute("paramtType", getTypeFromFullQualifiedName(parameter.getType()));
						operationElement.appendChild(paramElement);
						//if (parent)
							//classSet.add(parameter.getType());
					}
				}
			}
			
			if (operation.getReturnParameter() != null){
				Element returnElement = document.createElement("Parameter");
				UMLParameter parameter = operation.getReturnParameter();
				returnElement.setAttribute("ID", Long.toString(parameter.getID()));
				returnElement.setAttribute("paramName", parameter.getName());
				returnElement.setAttribute("paramKind", parameter.getKind());
				returnElement.setAttribute("paramType", getTypeFromFullQualifiedName(parameter.getType()));
				operationElement.appendChild(returnElement);
				//if (parent)
					//classSet.add(parameter.getType());
			}
			classElement.appendChild(operationElement);
		}
		List<UMLAttribute> attributes = umlClass.getAttributes();
		Collections.sort(attributes, new NameComparator());
		for(UMLAttribute attribute : attributes){
			Element attElement = document.createElement("Attribute");
			attElement.setAttribute("attrName", attribute.getName());
			attElement.setAttribute("type", getTypeFromFullQualifiedName(attribute.getType()));
			attElement.setAttribute("ID", Long.toString(attribute.getID()));
			attElement.setAttribute("visibility", attribute.getVisibility());
			classElement.appendChild(attElement);
			
			//if (parent)
				//classSet.add(attribute.getType());
		}
		parent = false;
		return classElement;
	}

}

