package generator.xml;


import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.w3c.dom.Element;
import gr.uom.java.xmi.UMLGeneralization;
import gr.uom.java.xmi.UMLModel;

public class XMLGeneralizationGenerator extends XMLGenerator {
	List<UMLGeneralization> generalizationList;
	Set<String> visited = new HashSet<String>();
	Map<String, List<String>> genRelationsMap = new HashMap<String, List<String>>();

	public XMLGeneralizationGenerator(UMLModel aModel, String element) {
		super(aModel, element);
		classes = model.getClassList();
		generalizationList = model.getGeneralizationList();
		initializeGenRelationsMap();
		
		Element root = document.createElement("VirtualRoot");
		root.setAttribute("name", "VirtualRoot");
		for(int i = 0; i < generalizationList.size(); i++){
			if (!visited.contains(generalizationList.get(i).getChild())){
				Element genElement = inherits(generalizationList.get(i).getChild(), false, false);
				if (genElement != null)
					root.appendChild(genElement);
			}
		}
		document.appendChild(root);	
	}
	
	private Element inherits(String parentID, boolean elementFlag, boolean isChild){
		visited.add(parentID);
		Element parentElement;
		if (isChild)
			parentElement = document.createElement("SuperClass");
		else 
			parentElement = document.createElement("Class");
		parentElement.setAttribute("name", parentID);
		if (parentID.equals(element)) elementFlag = true;
			
		if(!genRelationsMap.containsKey(parentID)) return parentElement;
		if (genRelationsMap.get(parentID) != null){
			List<String> children = genRelationsMap.get(parentID);
			for(String child : children){
				if (element == null) elementFlag = true; 
				else if (child.equals(element)) elementFlag = true;
				Element e = inherits(child, elementFlag, true);
				if (e != null){
					if(parentID.equals(element) && !child.equals(element))
						parentElement.appendChild(e);
				}
			}
		}
		if (elementFlag)
			return parentElement;
		return null;
	}
	private void initializeGenRelationsMap(){
		for(int i = 0; i < generalizationList.size(); i++)
			if(genRelationsMap.containsKey(generalizationList.get(i).getChild()))
			{
				List<String> parents = genRelationsMap.get(generalizationList.get(i).getChild());
				parents.add(generalizationList.get(i).getParent());
				genRelationsMap.put(generalizationList.get(i).getChild(), parents);
			}
			else{
				List<String> parents = new ArrayList<String>();
				parents.add(generalizationList.get(i).getParent());
				genRelationsMap.put(generalizationList.get(i).getChild(), parents);
			}
	}

}
