package generator.xml;


import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Set;
import org.w3c.dom.Element;

import gr.uom.java.xmi.AccessedMember;
import gr.uom.java.xmi.FieldAccess;
import gr.uom.java.xmi.MethodCall;
import gr.uom.java.xmi.UMLClass;
import gr.uom.java.xmi.UMLModel;
import gr.uom.java.xmi.UMLOperation;
import gr.uom.java.xmi.UMLParameter;

public class XMLUsageGenerator extends XMLGenerator {
	
	Set<AccessedMember> accessedMembersSet;
	boolean parent;
	//make as Singleton
	
	public XMLUsageGenerator(UMLModel aModel, String element) {
		super(aModel, element);

		classes = model.getClassList();
		Element root = document.createElement("VirtualRoot");
		root.setAttribute("name", "VirtualRoot");
		if (element != null){
					for(int i = 0; i < classes.size(); i++){
						if (classes.get(i).getName().equals(element)){
							root.appendChild(uses((UMLClass)classes.get(i)));
						}
					}		
			}
		else 
			for(int i = 0; i < classes.size(); i++){
				root.appendChild(uses((UMLClass)classes.get(i)));
			}
		document.appendChild(root);	
		
	}

	
	private Element uses(UMLClass umlClass){
		Element classElement = document.createElement("Class");
		classElement.setAttribute("ID", Long.toString(umlClass.getID()));
		classElement.setAttribute("className", umlClass.getName());
		classElement.setAttribute("isAbstract", Boolean.toString(umlClass.isAbstract()));
		classElement.setAttribute("isInterface", Boolean.toString(umlClass.isInterface()));
		
		List<UMLOperation> operations = umlClass.getOperations();
		Collections.sort(operations, new NameComparator());
		for(UMLOperation operation : operations){
			Element operationElement = document.createElement("Operation");
			operationElement.setAttribute("operationName", operation.getName());
			operationElement.setAttribute("visibility", operation.getVisibility());
			operationElement.setAttribute("ID", Long.toString(operation.getID()));
			//operationElement.setAttribute("isAbstract", Boolean.toString(operation.isAbstract()));
			
			List<UMLParameter> parameters = operation.getParameters();
			Collections.sort(parameters, new NameComparator());
			
			if(parameters != null){
				int k = 0;
				for (UMLParameter parameter : parameters){
					String counter = null;	
					if(k < 10)
						counter = "0" + k;
					else
						counter = String.valueOf(k);
						operationElement.setAttribute("param" + counter, parameter.getType());
						k++;
						Element paramElement = document.createElement("Parameter");
						if(!parameter.getKind().equals("return"))
							paramElement.setAttribute("paramName", parameter.getName());
						paramElement.setAttribute("paramKind", parameter.getKind());
						paramElement.setAttribute("paramtType", parameter.getType());
						operationElement.appendChild(paramElement);
					}
				}
			
			
			/*if (operation.getReturnParameter() != null){
				Element returnElement = document.createElement("returnParameter");
				UMLParameter parameter = operation.getReturnParameter();
				returnElement.setAttribute("ID", Long.toString(parameter.getID()));
				returnElement.setAttribute("paramName", parameter.getName());
				returnElement.setAttribute("kind", parameter.getKind());
				returnElement.setAttribute("type", parameter.getType());
				operationElement.appendChild(returnElement);
				//if (parent)
					//classSet.add(parameter.getType());
			}*/
			
			accessedMembersSet = operation.getAccessedMembers();
			List<AccessedMember> accessedMembers= new ArrayList(accessedMembersSet);
			Collections.sort(accessedMembers, new NameComparator());
			for(AccessedMember accessedMember : accessedMembers){
				if(accessedMember instanceof FieldAccess){
					Element fieldElement = document.createElement("FieldAccess");
					fieldElement.setAttribute("ID", Long.toString(((FieldAccess) accessedMember).getID()));
					fieldElement.setAttribute("fieldName", ((FieldAccess) accessedMember).getOwnerClassName()+"."+((FieldAccess) accessedMember).getFieldName());
					fieldElement.setAttribute("ClassType", ((FieldAccess) accessedMember).getClassType());
					fieldElement.setAttribute("OwnerClassName", ((FieldAccess) accessedMember).getOwnerClassName());
					operationElement.appendChild(fieldElement);
				}
				else if(accessedMember instanceof MethodCall){
					Element methodCallElement;
					if(((MethodCall) accessedMember).isConstructorCall()){
						methodCallElement = document.createElement("ConstructorCall");
						methodCallElement.setAttribute("ConstructorCallName", ((MethodCall) accessedMember).getOriginClassName());
					}
					else{
						methodCallElement = document.createElement("MethodCall");
						methodCallElement.setAttribute("methodCallName", ((MethodCall) accessedMember).getOriginClassName()+"."+((MethodCall) accessedMember).getMethodName());
					}
					methodCallElement.setAttribute("ID", Long.toString(((MethodCall) accessedMember).getID()));
					methodCallElement.setAttribute("originClassName", ((MethodCall) accessedMember).getOriginClassName());
					List<String> paramList =((MethodCall) accessedMember).getParamTypeList();
					int kn=0;
					if(((MethodCall) accessedMember).getReturnType() != null && 
							!((MethodCall) accessedMember).getReturnType().equals("") ){
						methodCallElement.setAttribute("arg" + (kn+1), ((MethodCall) accessedMember).getReturnType());
						kn++;
						Element returnElement = document.createElement("Argument");
						returnElement.setAttribute("argKind", "return");
						returnElement.setAttribute("argType",((MethodCall) accessedMember).getReturnType());
						methodCallElement.appendChild(returnElement);
					}
					for(String param : paramList){
						methodCallElement.setAttribute("arg" + (kn+1), param);
						kn++;
						Element paramElement = document.createElement("Argument");
						paramElement.setAttribute("argType", param);
						paramElement.setAttribute("argKind", "in");
						methodCallElement.appendChild(paramElement);
					}
					
					operationElement.appendChild(methodCallElement);
				}
				//Element memberElement = document.createElement(accessedMember.)
			}
			classElement.appendChild(operationElement);	
		}
		
		return classElement;
	}
	
}

