package generator.xml;

import java.io.StringWriter;
import java.util.ArrayList;
import java.util.List;

import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import model.diff.XMLStringPair;

import org.w3c.dom.Document;

import gr.uom.java.xmi.UMLModel;

public class XMLWriter {
	
	List<XMLStringPair> xmls = new ArrayList<XMLStringPair>();
	
	public XMLWriter(UMLModel model1, UMLModel model2, String element,
			boolean enableContainment,
			boolean enableGeneralization,
			boolean enableRealization,
			boolean enableUsage) {
		
		if(enableContainment) {
			XMLStringPair xmlPair = new XMLStringPair();
			xmlPair.setType("cont");
			xmlPair.setFirstXMLString(generateXMLContainment(model1, element));
			xmlPair.setSecondXMLString(generateXMLContainment(model2, element));
			xmls.add(xmlPair);
		}
		if(enableGeneralization) {
			XMLStringPair xmlPair = new XMLStringPair();
			xmlPair.setType("gener");
			xmlPair.setFirstXMLString(generateXMLGeneralization(model1, element));
			xmlPair.setSecondXMLString(generateXMLGeneralization(model2, element));
			xmls.add(xmlPair);
		}
		if(enableRealization) {
			XMLStringPair xmlPair = new XMLStringPair();
			xmlPair.setType("realiz");
			xmlPair.setFirstXMLString(generateXMLRealization(model1, element));
			xmlPair.setSecondXMLString(generateXMLRealization(model2, element));
			xmls.add(xmlPair);
		}
		if(enableUsage) {
			XMLStringPair xmlPair = new XMLStringPair();
			xmlPair.setType("usage");
			xmlPair.setFirstXMLString(generateXMLUsage(model1, element));
			xmlPair.setSecondXMLString(generateXMLUsage(model2, element));
			xmls.add(xmlPair);
		}
	}
	
	private String generateXMLContainment(UMLModel model, String element){
			XMLGenerator classCont = new XMLContainmentGenerator(model, element);
			Document document = classCont.getXMLDocument();
			return createXMLFile(document);
	}
	
	private String generateXMLGeneralization(UMLModel model, String element){
			XMLGenerator classGener = new XMLGeneralizationGenerator(model, element);
			Document document = classGener.getXMLDocument();
			return createXMLFile(document);
	}
	
	private String generateXMLRealization(UMLModel model, String element){
			XMLGenerator classRealiz = new XMLRealizationGenerator(model, element);
			Document document = classRealiz.getXMLDocument();
			return createXMLFile(document);
	 }
	private String generateXMLUsage(UMLModel model, String element){
			XMLGenerator usage = new XMLUsageGenerator(model, element);
			Document document = usage.getXMLDocument();
			return createXMLFile(document);
	}
	
	private String createXMLFile(Document document){
		 TransformerFactory transformerFactory = TransformerFactory.newInstance();
	     Transformer transformer;
		 try {
			 transformer = transformerFactory.newTransformer();
		     DOMSource source = new DOMSource(document);
		
 		     StringWriter xmlOut = new StringWriter();
 		     StreamResult result = new StreamResult(xmlOut);
 
		   //  System.out.println("Generating new xml document");
		     try {
				transformer.transform(source, result);
				return xmlOut.getBuffer().toString();
		
			} catch (TransformerException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		} catch (TransformerConfigurationException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return null;
	}

	public List<XMLStringPair> getXMLPairs(){
		return this.xmls;
	}
	
}

