package gr.uom.java.xmi;

import java.util.ArrayList;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Set;

public class UMLOperation implements Comparable<UMLOperation> {
	private String name;
	private String xmiID;
	private String visibility;
	private boolean isAbstract;
	private List<UMLParameter> parameters;
	private String className;
	private Set<AccessedMember> accessedMembers;
	private long ID;


	
	public UMLOperation(String name, String xmiID, long ID) {
        this.name = name;
        this.xmiID = xmiID;
        this.parameters = new ArrayList<UMLParameter>();
        this.accessedMembers = new LinkedHashSet<AccessedMember>();
        this.ID = ID;
    }
	
	public long getID(){
		return ID;
	}

	public void addAccessedMember(AccessedMember member) {

		this.accessedMembers.add(member);

	}
	
	public String getQualifiedName(){
		 String params = "";
		 for(UMLParameter parameter : parameters){
			 params += parameter.getName() + ", ";
		 }
		 return (this.name + "(" + params + ")");
			 
	}

	public List<String> getParameterTypeList() {

		List<String> parameterTypeList = new ArrayList<String>();

		for(UMLParameter parameter : parameters) {

			if(!parameter.getKind().equals("return"))

				parameterTypeList.add(parameter.getType());

		}

		return parameterTypeList;

	}


	public Set<AccessedMember> getAccessedMembers() {

		return this.accessedMembers;

	}



	public void setAccessedMembers(Set<AccessedMember> accessedMembers) {

		this.accessedMembers = accessedMembers;

	}
	public String getName() {
		return name;
	}

	public String getXmiID() {
		return xmiID;
	}

	public String getVisibility() {
		return visibility;
	}

	public void setVisibility(String visibility) {
		this.visibility = visibility;
	}

	public boolean isAbstract() {
		return isAbstract;
	}

	public void setAbstract(boolean isAbstract) {
		this.isAbstract = isAbstract;
	}

	public String getClassName() {
		return className;
	}

	public void setClassName(String className) {
		this.className = className;
	}

	public void addParameter(UMLParameter parameter) {
		this.parameters.add(parameter);
	}

	public List<UMLParameter> getParameters() {
		return parameters;
	}

	public UMLParameter getReturnParameter() {
		for(UMLParameter parameter : parameters) {
			if(parameter.getKind().equals("return"))
				return parameter;
		}
		return null;
	}

	public boolean equalReturnParameter(UMLOperation operation) {
		UMLParameter thisReturnParameter = this.getReturnParameter();
		UMLParameter otherReturnParameter = operation.getReturnParameter();
		if(thisReturnParameter != null && otherReturnParameter != null)
			return thisReturnParameter.equals(otherReturnParameter);
		else if(thisReturnParameter == null && otherReturnParameter == null)
			return true;
		else
			return false;
	}

	public boolean equals(Object o) {
		if(this == o) {
            return true;
        }
		
		if(o instanceof UMLOperation) {
			UMLOperation operation = (UMLOperation)o;
			return this.name.equals(operation.name) &&
				this.visibility.equals(operation.visibility) &&
				this.isAbstract == operation.isAbstract &&
				this.parameters.equals(operation.parameters);
		}
		return false;
	}

	public String toString() {
		StringBuilder sb = new StringBuilder();
		sb.append(visibility);
		sb.append(" ");
		if(isAbstract) {
			sb.append("abstract");
			sb.append(" ");
		}
		sb.append(name);
		UMLParameter returnParameter = getReturnParameter();
		List<UMLParameter> parameters = new ArrayList<UMLParameter>(this.parameters);
		parameters.remove(returnParameter);
		sb.append("(");
		for(int i=0; i<parameters.size(); i++) {
			UMLParameter parameter = parameters.get(i);
			if(parameter.getKind().equals("in")) {
				sb.append(parameter);
				if(i < parameters.size()-1)
					sb.append(", ");
			}
		}
		sb.append(")");
		if(returnParameter != null) {
			sb.append(" : ");
			sb.append(returnParameter.toString());
		}
		return sb.toString();
	}

	 public int compareTo(UMLOperation operation) {
	        return this.toString().compareTo(operation.toString());
	    }
}
