package gr.uom.java.xmi.diff;

import gr.uom.java.xmi.UMLAction;
import gr.uom.java.xmi.UMLAssociation;
import gr.uom.java.xmi.UMLCallAction;
import gr.uom.java.xmi.UMLClass;
import gr.uom.java.xmi.UMLDependency;
import gr.uom.java.xmi.UMLGeneralization;
import gr.uom.java.xmi.UMLMessage;
import gr.uom.java.xmi.UMLOperation;
import gr.uom.java.xmi.UMLParameter;
import gr.uom.java.xmi.UMLRealization;

import java.util.ArrayList;
import java.util.Collections;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Set;

public class UMLModelDiff {
	private List<UMLClass> addedClasses;
	private List<UMLClass> removedClasses;
	private List<UMLGeneralization> addedGeneralizations;
	private List<UMLGeneralization> removedGeneralizations;
	private List<UMLAssociation> addedAssociations;
	private List<UMLAssociation> removedAssociations;
	private List<UMLDependency> addedDependencies;
	private List<UMLDependency> removedDependencies;
	private List<UMLRealization> addedRealizations;
	private List<UMLRealization> removedRealizations;
	private List<UMLClassDiff> commonClassDiffList;
	private List<UMLCollaborationDiff> collaborationDiffList;
	
	public UMLModelDiff() {
		this.addedClasses = new ArrayList<UMLClass>();
		this.removedClasses = new ArrayList<UMLClass>();
		this.addedGeneralizations = new ArrayList<UMLGeneralization>();
		this.removedGeneralizations = new ArrayList<UMLGeneralization>();
		this.addedAssociations = new ArrayList<UMLAssociation>();
		this.removedAssociations = new ArrayList<UMLAssociation>();
		this.addedDependencies = new ArrayList<UMLDependency>();
		this.removedDependencies = new ArrayList<UMLDependency>();
		this.addedRealizations = new ArrayList<UMLRealization>();
		this.removedRealizations = new ArrayList<UMLRealization>();
		this.commonClassDiffList = new ArrayList<UMLClassDiff>();
		this.collaborationDiffList = new ArrayList<UMLCollaborationDiff>();
	}
	
	public void reportAddedClass(UMLClass umlClass) {
		this.addedClasses.add(umlClass);
	}

	public void reportRemovedClass(UMLClass umlClass) {
		this.removedClasses.add(umlClass);
	}

	public void reportAddedGeneralization(UMLGeneralization umlGeneralization) {
		this.addedGeneralizations.add(umlGeneralization);
	}

	public void reportRemovedGeneralization(UMLGeneralization umlGeneralization) {
		this.removedGeneralizations.add(umlGeneralization);
	}

	public void reportAddedAssociation(UMLAssociation umlAssociation) {
		this.addedAssociations.add(umlAssociation);
	}

	public void reportRemovedAssociation(UMLAssociation umlAssociation) {
		this.removedAssociations.add(umlAssociation);
	}

	public void reportAddedDependency(UMLDependency umlDependency) {
		this.addedDependencies.add(umlDependency);
	}

	public void reportRemovedDependency(UMLDependency umlDependency) {
		this.removedDependencies.add(umlDependency);
	}

	public void reportAddedRealization(UMLRealization umlRealization) {
		this.addedRealizations.add(umlRealization);
	}

	public void reportRemovedRealization(UMLRealization umlRealization) {
		this.removedRealizations.add(umlRealization);
	}

	public void addUMLClassDiff(UMLClassDiff classDiff) {
		this.commonClassDiffList.add(classDiff);
	}

	public void addUMLCollaborationDiff(UMLCollaborationDiff collaborationDiff) {
		this.collaborationDiffList.add(collaborationDiff);
	}

	public boolean isAddedClass(String className) {
		for(UMLClass umlClass : addedClasses) {
			if(umlClass.getName().equals(className))
				return true;
		}
		return false;
	}

	public boolean isRemovedClass(String className) {
		for(UMLClass umlClass : removedClasses) {
			if(umlClass.getName().equals(className))
				return true;
		}
		return false;
	}

	public void checkForGeneralizationChanges() {
		Set<UMLGeneralization> removedGeneralizationsToBeRemoved = new LinkedHashSet<UMLGeneralization>();
		Set<UMLGeneralization> addedGeneralizationsToBeRemoved = new LinkedHashSet<UMLGeneralization>();
		for(UMLGeneralization removedGeneralization : removedGeneralizations) {
			for(UMLGeneralization addedGeneralization : addedGeneralizations) {
				if(removedGeneralization.getChild().equals(addedGeneralization.getChild())) {
					UMLGeneralizationDiff generalizationDiff = new UMLGeneralizationDiff(removedGeneralization, addedGeneralization);
					addedGeneralizationsToBeRemoved.add(addedGeneralization);
					removedGeneralizationsToBeRemoved.add(removedGeneralization);
					//generalizationDiffList.add(generalizationDiff);
					break;
				}
			}
		}
		this.addedGeneralizations.removeAll(addedGeneralizationsToBeRemoved);
		this.removedGeneralizations.removeAll(removedGeneralizationsToBeRemoved);
	}

	public List<UMLClass> getAddedClasses() {
		return addedClasses;
	}

	public List<UMLClass> getRemovedClasses() {
		return removedClasses;
	}

	public List<UMLGeneralization> getAddedGeneralizations() {
		return addedGeneralizations;
	}

	public List<UMLGeneralization> getRemovedGeneralizations() {
		return removedGeneralizations;
	}

	public List<UMLAssociation> getAddedAssociations() {
		return addedAssociations;
	}

	public List<UMLAssociation> getRemovedAssociations() {
		return removedAssociations;
	}

	public List<UMLDependency> getAddedDependencies() {
		return addedDependencies;
	}

	public List<UMLDependency> getRemovedDependencies() {
		return removedDependencies;
	}

	public List<UMLRealization> getAddedRealizations() {
		return addedRealizations;
	}

	public List<UMLRealization> getRemovedRealizations() {
		return removedRealizations;
	}

	public List<UMLClassDiff> getCommonClassDiffList() {
		return commonClassDiffList;
	}

	public List<UMLCollaborationDiff> getCollaborationDiffList() {
		return collaborationDiffList;
	}

	

/*
	private List<MoveOperationRefactoring> identifyMoveOperationRefactorings() {
		List<MoveOperationRefactoring> refactorings = new ArrayList<MoveOperationRefactoring>();
		List<UMLOperation> addedOperations = getAddedOperationsInCommonClasses();
		List<UMLOperation> removedOperations = getRemovedOperationsInCommonClasses();
		
		List<UMLOperation> addedOperationsWithIdenticalNameInRemovedOperations = new ArrayList<UMLOperation>();
		for(UMLOperation addedOperation : addedOperations) {
			for(UMLOperation removedOperation : removedOperations) {
				if(addedOperation.getName().equals(removedOperation.getName()) &&
						addedOperation.equalReturnParameter(removedOperation) &&
						addedOperation.isAbstract() == removedOperation.isAbstract())
					addedOperationsWithIdenticalNameInRemovedOperations.add(addedOperation);
			}
		}
		for(UMLMessage message : getAddedMessages()) {
			UMLAction action = message.getAction();
			if(action instanceof UMLCallAction) {
				UMLCallAction callAction = (UMLCallAction)action;
				UMLOperation callActionOperation = callAction.getOperation();
				if(callActionOperation != null &&
						addedOperationsWithIdenticalNameInRemovedOperations.contains(callActionOperation)) {
					MoveOperationRefactoring moveOperation = new MoveOperationRefactoring(callActionOperation,
							message.getSender().getBase(), message.getReceiver().getBase());
					refactorings.add(moveOperation);
				}
			}
		}
		return refactorings;
	}
*/
	private List<UMLMessage> getAddedMessages() {
		List<UMLMessage> addedMessages = new ArrayList<UMLMessage>();
		for(UMLCollaborationDiff collaborationDiff : collaborationDiffList) {
			addedMessages.addAll(collaborationDiff.getAddedMessages());
		}
		return addedMessages;
	}

	private List<UMLOperation> getAddedOperationsInCommonClasses() {
		List<UMLOperation> addedOperations = new ArrayList<UMLOperation>();
		for(UMLClassDiff classDiff : commonClassDiffList) {
			addedOperations.addAll(classDiff.getAddedOperations());
		}
		return addedOperations;
	}

	private List<UMLOperation> getRemovedOperationsInCommonClasses() {
		List<UMLOperation> removedOperations = new ArrayList<UMLOperation>();
		for(UMLClassDiff classDiff : commonClassDiffList) {
			removedOperations.addAll(classDiff.getRemovedOperations());
		}
		return removedOperations;
	}

	

	public boolean isEmpty() {
		return addedClasses.isEmpty() && removedClasses.isEmpty() &&
		addedGeneralizations.isEmpty() && removedGeneralizations.isEmpty() &&
		addedAssociations.isEmpty() && removedAssociations.isEmpty() &&
		addedDependencies.isEmpty() && removedDependencies.isEmpty() &&
		addedRealizations.isEmpty() && removedRealizations.isEmpty() &&
		emptyClassDiffs() && emptyCollaborationDiffs();
	}
	private boolean emptyClassDiffs() {
		for(UMLClassDiff classDiff : commonClassDiffList) {
			if(!classDiff.isEmpty())
				return false;
		}
		return true;
	}
	private boolean emptyCollaborationDiffs() {
		for(UMLCollaborationDiff collaborationDiff : collaborationDiffList) {
			if(!collaborationDiff.isEmpty())
				return false;
		}
		return true;
	}
	public String toString() {
		StringBuilder sb = new StringBuilder();
		//if(!addedClasses.isEmpty() || !removedClasses.isEmpty())
			//sb.append("--class level edits--").append("\n");
		Collections.sort(removedClasses);
		for(UMLClass umlClass : removedClasses) {
			sb.append("class " + umlClass + " removed").append("\n");
		}
		Collections.sort(addedClasses);
		for(UMLClass umlClass : addedClasses) {
			sb.append("class " + umlClass + " added").append("\n");
		}
		//if(!addedGeneralizations.isEmpty() || !removedGeneralizations.isEmpty())// || !generalizationDiffList.isEmpty())
			//sb.append("--generalization edits--").append("\n");
		Collections.sort(removedGeneralizations);
		for(UMLGeneralization umlGeneralization : removedGeneralizations) {
			sb.append("generalization " + umlGeneralization + " removed").append("\n");
		}
		Collections.sort(addedGeneralizations);
		for(UMLGeneralization umlGeneralization : addedGeneralizations) {
			sb.append("generalization " + umlGeneralization + " added").append("\n");
		}
		/*Collections.sort(generalizationDiffList);
		for(UMLGeneralizationDiff generalizationDiff : generalizationDiffList) {
			sb.append(generalizationDiff);
		}*/
		//if(!addedAssociations.isEmpty() || !removedAssociations.isEmpty())
			//sb.append("--association edits--").append("\n");
		Collections.sort(removedAssociations);
		for(UMLAssociation umlAssociation : removedAssociations) {
			sb.append("association " + umlAssociation + " removed").append("\n");
		}
		Collections.sort(addedAssociations);
		for(UMLAssociation umlAssociation : addedAssociations) {
			sb.append("association " + umlAssociation + " added").append("\n");
		}
		//if(!addedDependencies.isEmpty() || !removedDependencies.isEmpty())
			//sb.append("--dependency edits--").append("\n");
		Collections.sort(removedDependencies);
		for(UMLDependency umlDependency : removedDependencies) {
			sb.append("dependency " + umlDependency + " removed").append("\n");
		}
		Collections.sort(addedDependencies);
		for(UMLDependency umlDependency : addedDependencies) {
			sb.append("dependency " + umlDependency + " added").append("\n");
		}
		//if(!addedRealizations.isEmpty() || !removedRealizations.isEmpty())
			//sb.append("--realization edits--").append("\n");
		Collections.sort(removedRealizations);
		for(UMLRealization umlRealization : removedRealizations) {
			sb.append("realization " + umlRealization + " removed").append("\n");
		}
		Collections.sort(addedRealizations);
		for(UMLRealization umlRealization : addedRealizations) {
			sb.append("realization " + umlRealization + " added").append("\n");
		}
		//if(!commonClassDiffList.isEmpty())
			//sb.append("--edits between common classes--").append("\n");
		Collections.sort(commonClassDiffList);
		for(UMLClassDiff classDiff : commonClassDiffList) {
			sb.append(classDiff);
		}
		//if(!collaborationDiffList.isEmpty())
			//sb.append("--edits between collaborations--").append("\n");
		for(UMLCollaborationDiff collaborationDiff : collaborationDiffList) {
			sb.append(collaborationDiff);
		}
		return sb.toString();
	}
}
