package gr.uom.java.xmi.diff;

import gr.uom.java.xmi.AccessedMember;
import gr.uom.java.xmi.FieldAccess;
import gr.uom.java.xmi.MethodCall;
import gr.uom.java.xmi.UMLOperation;
import gr.uom.java.xmi.UMLParameter;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Set;

public class UMLOperationBodyDiff implements Comparable<UMLOperationBodyDiff> {
	private String operation;
	private List<AccessedMember> addedAccessedMembers;
	private List<AccessedMember> removedAccessedMembers;
	private List<AccessedMemberDiff> changedAccessedMembers;
	
	public UMLOperationBodyDiff(String operation) {
		this.operation = operation;
		this.addedAccessedMembers = new ArrayList<AccessedMember>();
		this.removedAccessedMembers = new ArrayList<AccessedMember>();
		this.changedAccessedMembers = new ArrayList<AccessedMemberDiff>();
	}

	public UMLOperationBodyDiff(UMLOperation operation1, UMLOperation operation2) {
		StringBuilder sb = new StringBuilder();
		sb.append(operation1.getName());
		String returnType = null;
		sb.append("(");
		for(int i=0; i<operation1.getParameters().size(); i++) {
			UMLParameter parameter = operation1.getParameters().get(i);
			if(parameter.getKind().equals("return")) {
				returnType = parameter.getType();
			}
			else if(parameter.getKind().equals("in")) {
				sb.append(parameter.getType());
				if(i < operation1.getParameters().size()-1)
					sb.append(", ");
			}
		}
		sb.append(")");
		if(returnType != null) {
			sb.append(" : ");
			sb.append(returnType);
		}
		this.operation = sb.toString();
		this.addedAccessedMembers = new ArrayList<AccessedMember>();
		this.removedAccessedMembers = new ArrayList<AccessedMember>();
		this.changedAccessedMembers = new ArrayList<AccessedMemberDiff>();
		Set<AccessedMember> accessedMembers1 = operation1.getAccessedMembers();
		Set<AccessedMember> accessedMembers2 = operation2.getAccessedMembers();
		for(AccessedMember accessedMember : accessedMembers1) {
			if(!accessedMembers2.contains(accessedMember))
				removedAccessedMembers.add(accessedMember);
		}
		for(AccessedMember accessedMember : accessedMembers2) {
			if(!accessedMembers1.contains(accessedMember))
				addedAccessedMembers.add(accessedMember);
		}
	}

	public UMLOperationBodyDiff(UMLOperation operation2, MethodCall call, int i) {
		// TODO Auto-generated constructor stub
	}

	public String getOperation() {
		return operation;
	}

	public void reportAddedAccessedMember(AccessedMember member) {
		this.addedAccessedMembers.add(member);
	}

	public void reportRemovedAccessedMember(AccessedMember member) {
		this.removedAccessedMembers.add(member);
	}

	public void reportChangedAccessedMember(AccessedMemberDiff diff) {
		this.changedAccessedMembers.add(diff);
	}
	public List<AccessedMember> getAddedAccessedMembers() {
		return addedAccessedMembers;
	}

	public List<AccessedMember> getRemovedAccessedMembers() {
		return removedAccessedMembers;
	}

	public List<AccessedMemberDiff> getChangedAccessedMembers() {
		return changedAccessedMembers;
	}

	public List<MethodCall> getAddedMethodCalls() {
		List<MethodCall> addedMethodCalls = new ArrayList<MethodCall>();
		for(AccessedMember accessedMember : addedAccessedMembers) {
			if(accessedMember instanceof MethodCall)
				addedMethodCalls.add((MethodCall)accessedMember);
		}
		return addedMethodCalls;
	}

	public boolean isEmpty() {
		return addedAccessedMembers.isEmpty() && removedAccessedMembers.isEmpty() && changedAccessedMembers.isEmpty();
	}

	public String toString() {
		StringBuilder sb = new StringBuilder();
		if(!isEmpty())
			sb.append("\t").append("operation ").append(operation).append("\n");
		Collections.sort(removedAccessedMembers);
		for(AccessedMember accessedMember : removedAccessedMembers) {
			String type = null;
			if(accessedMember instanceof MethodCall)
				type = "method call";
			else if(accessedMember instanceof FieldAccess)
				type = "field access";
			sb.append("\t\t").append(type + " " + accessedMember + " removed").append("\n");
		}
		Collections.sort(addedAccessedMembers);
		for(AccessedMember accessedMember : addedAccessedMembers) {
			String type = null;
			if(accessedMember instanceof MethodCall)
				type = "method call";
			else if(accessedMember instanceof FieldAccess)
				type = "field access";
			sb.append("\t\t").append(type + " " + accessedMember + " added").append("\n");
		}
		for(AccessedMemberDiff diff : changedAccessedMembers) {
			sb.append(diff.toString());
		}
		return sb.toString();
	}

	public int compareTo(UMLOperationBodyDiff operationBodyDiff) {
		return this.operation.compareTo(operationBodyDiff.operation);
	}
}
