package model.diff;
import gr.uom.java.xmi.AccessedMember;
import gr.uom.java.xmi.FieldAccess;
import gr.uom.java.xmi.MethodCall;
import gr.uom.java.xmi.diff.AccessedMemberDiff;
import gr.uom.java.xmi.diff.UMLClassDiff;
import gr.uom.java.xmi.diff.UMLOperationBodyDiff;

import java.io.BufferedReader;
import java.io.ByteArrayInputStream;
import java.io.DataInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.List;


public class EditScriptParser {
	private List<String> lines;
	private String className;
	private UMLClassDiff classDiff;
	private UMLOperationBodyDiff currentOperationBodyDiff;
	private AccessedMember currentAccessedMember;
	
	private int parsingStatus;
	private static final int INSERT = 1;
	private static final int DELETE = 2;
	//private String previousState;
	private boolean isChangePropagated = false;
	
	public EditScriptParser(byte[] editScriptResult, String className) {
		this.className = className;
		this.classDiff = new UMLClassDiff(className);
		this.lines = new ArrayList<String>();
		try {
			//FileInputStream fstream = new FileInputStream(file);
			ByteArrayInputStream bstream = new ByteArrayInputStream(editScriptResult);
			DataInputStream in = new DataInputStream(bstream);
			BufferedReader br = new BufferedReader(new InputStreamReader(in));
			String strLine;
			while ((strLine = br.readLine()) != null) {
				if(!strLine.equals(""))
					lines.add(strLine);
			}
			in.close();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	public void parse() {
		for(int i=lines.size()-1; i>=0; i--) {
			String line = lines.get(i);
			if(line.startsWith("change-propagated")) {
				processChangePropagatedLine(line);
			}
			else if(line.startsWith("insert")) {
				parsingStatus = INSERT;
				processInsertDelete(line);				
			}
			else if(line.startsWith("delete")) {
				parsingStatus = DELETE;
				processInsertDelete(line);				
			}
			else if(line.startsWith("change(")) {
				processChange(line);
			}
		}
		if(currentOperationBodyDiff != null && !currentOperationBodyDiff.isEmpty()) {
			/*if(currentAccessedMember != null) {
				if(parsingStatus == INSERT)
					currentOperationBodyDiff.reportAddedAccessedMember(currentAccessedMember);
				else if(parsingStatus == DELETE)
					currentOperationBodyDiff.reportRemovedAccessedMember(currentAccessedMember);
			}*/
			classDiff.addOperationBodyDiff(currentOperationBodyDiff);
		}
	}
	
	private void processChange(String line) {
		if(line.contains("{MethodCall}")) {
			String method1 = line.substring(line.indexOf("{MethodCall}")+13,line.indexOf("=>"));
			String method2 = line.substring(line.lastIndexOf("{MethodCall}")+13, line.length());
			MethodCall methodCall1 = generateMethodCall(method1);
			MethodCall methodCall2 = generateMethodCall(method2);
			AccessedMemberDiff diff = new AccessedMemberDiff(methodCall1, methodCall2);
			if(isChangePropagated && currentOperationBodyDiff != null) {
				currentOperationBodyDiff.reportChangedAccessedMember(diff);
			}
		}
		else if(line.contains("{ConstructorCall}")) {
			String method1 = line.substring(line.indexOf("{ConstructorCall}")+18,line.indexOf("=>"));
			String method2 = line.substring(line.lastIndexOf("{ConstructorCall}")+18, line.length());
			MethodCall methodCall1 = generateConstructorCall(method1);
			MethodCall methodCall2 = generateConstructorCall(method2);
			AccessedMemberDiff diff = new AccessedMemberDiff(methodCall1, methodCall2);
			if(isChangePropagated && currentOperationBodyDiff != null) {
				currentOperationBodyDiff.reportChangedAccessedMember(diff);
			}
		}
		else if(line.contains("{FieldAccess}")) {
			String field1 = line.substring(line.indexOf("{FieldAccess}")+14,line.indexOf("=>"));
			String field2 = line.substring(line.lastIndexOf("{FieldAccess}")+14, line.length());
			FieldAccess fieldAccess1 = generateFieldAccess(field1);
			FieldAccess fieldAccess2 = generateFieldAccess(field2);
			AccessedMemberDiff diff = new AccessedMemberDiff(fieldAccess1, fieldAccess2);
			if(isChangePropagated && currentOperationBodyDiff != null) {
				currentOperationBodyDiff.reportChangedAccessedMember(diff);
			}
		}
		else if(line.contains("{Operation}")) {
			isChangePropagated = false;
		}
	}
	private FieldAccess generateFieldAccess(String signature) {
		String subSignature = signature.substring(0, signature.lastIndexOf("("));
		String className = subSignature.substring(0, subSignature.lastIndexOf("."));
		String fieldName = subSignature.substring(subSignature.lastIndexOf(".")+1, subSignature.length());
		FieldAccess fieldAccess = new FieldAccess(className, fieldName);
		String arguments = signature.substring(signature.lastIndexOf("(")+1, signature.lastIndexOf(")"));
		String[] argumentTokens = arguments.split(",");
		for(int i=0; i<argumentTokens.length; i++) {
			String argument = argumentTokens[i];
			if(i == 0) {
				fieldAccess.setClassType(argument);
			}
		}
		return fieldAccess;
	}
	private MethodCall generateConstructorCall(String signature) {
		String subSignature = signature.substring(0, signature.lastIndexOf("("));
		String arguments = signature.substring(signature.lastIndexOf("(")+1, signature.lastIndexOf(")"));
		MethodCall methodCall = new MethodCall(subSignature, "");
		methodCall.setConstructorCall(true);
		String[] argumentTokens = arguments.split(",");
		for(int i=0; i<argumentTokens.length; i++) {
			String argument = argumentTokens[i];
			if(i == 0) {
				methodCall.setReturnType(argument);
			}
			else {
				methodCall.addParameter(argument);
			}
		}
		return methodCall;
	}
	private MethodCall generateMethodCall(String signature) {
		String subSignature = signature.substring(0, signature.lastIndexOf("("));
		String className = subSignature.substring(0, subSignature.lastIndexOf("."));
		String methodName = subSignature.substring(subSignature.lastIndexOf(".")+1, subSignature.length());
		MethodCall methodCall = new MethodCall(className, methodName);
		String arguments = signature.substring(signature.lastIndexOf("(")+1, signature.lastIndexOf(")"));
		String[] argumentTokens = arguments.split(",");
		for(int i=0; i<argumentTokens.length; i++) {
			String argument = argumentTokens[i];
			if(i == 0) {
				methodCall.setReturnType(argument);
			}
			else {
				methodCall.addParameter(argument);
			}
		}
		return methodCall;
	}
	private void processInsertDelete(String line) {
		if(line.contains("{MethodCall}")) {
			String signature = line.substring(line.lastIndexOf("{MethodCall}")+13, line.lastIndexOf("("));
			String arguments = line.substring(line.lastIndexOf("(")+1, line.lastIndexOf(")"));
			String className = signature.substring(0, signature.lastIndexOf("."));
			String methodName = signature.substring(signature.lastIndexOf(".")+1, signature.length());
			MethodCall methodCall = new MethodCall(className, methodName);
			String[] argumentTokens = arguments.split(",");
			for(int i=0; i<argumentTokens.length; i++) {
				String argument = argumentTokens[i];
				if(i == 0) {
					methodCall.setReturnType(argument);
				}
				else {
					methodCall.addParameter(argument);
				}
			}
			currentAccessedMember = methodCall;
			if(isChangePropagated && currentOperationBodyDiff != null) {
				if(parsingStatus == INSERT)
					currentOperationBodyDiff.reportAddedAccessedMember(currentAccessedMember);
				else if(parsingStatus == DELETE)
					currentOperationBodyDiff.reportRemovedAccessedMember(currentAccessedMember);
			}
			//previousState = "MethodCall";
		}
		else if(line.contains("{ConstructorCall}")) {
			String signature = line.substring(line.lastIndexOf("{ConstructorCall}")+18, line.lastIndexOf("("));
			String arguments = line.substring(line.lastIndexOf("(")+1, line.lastIndexOf(")"));
			//String className = signature.substring(0, signature.lastIndexOf("."));
			//String methodName = signature.substring(signature.lastIndexOf(".")+1, signature.length());
			MethodCall methodCall = new MethodCall(signature, "");
			methodCall.setConstructorCall(true);
			String[] argumentTokens = arguments.split(",");
			for(int i=0; i<argumentTokens.length; i++) {
				String argument = argumentTokens[i];
				if(i == 0) {
					methodCall.setReturnType(argument);
				}
				else {
					methodCall.addParameter(argument);
				}
			}
			currentAccessedMember = methodCall;
			if(isChangePropagated && currentOperationBodyDiff != null) {
				if(parsingStatus == INSERT)
					currentOperationBodyDiff.reportAddedAccessedMember(currentAccessedMember);
				else if(parsingStatus == DELETE)
					currentOperationBodyDiff.reportRemovedAccessedMember(currentAccessedMember);
			}
			//previousState = "ConstructorCall";
		}
		else if(line.contains("{FieldAccess}")) {
			String signature = line.substring(line.lastIndexOf("{FieldAccess}")+14, line.lastIndexOf("("));
			String className = signature.substring(0, signature.lastIndexOf("."));
			String fieldName = signature.substring(signature.lastIndexOf(".")+1, signature.length());
			FieldAccess fieldAccess = new FieldAccess(className, fieldName);
			String arguments = line.substring(line.lastIndexOf("(")+1, line.lastIndexOf(")"));
			String[] argumentTokens = arguments.split(",");
			for(int i=0; i<argumentTokens.length; i++) {
				String argument = argumentTokens[i];
				if(i == 0) {
					fieldAccess.setClassType(argument);
				}
			}
			currentAccessedMember = fieldAccess;
			if(isChangePropagated && currentOperationBodyDiff != null) {
				if(parsingStatus == INSERT)
					currentOperationBodyDiff.reportAddedAccessedMember(currentAccessedMember);
				else if(parsingStatus == DELETE)
					currentOperationBodyDiff.reportRemovedAccessedMember(currentAccessedMember);
			}
		}
		else if(line.contains("{Operation}")) {
			isChangePropagated = false;
		}
		/*else if(line.contains("{Argument}")) {
			String argument = line.substring(line.lastIndexOf("{Argument}")+11, line.length());
			if(previousState.equals("MethodCall") || previousState.equals("ConstructorCall")) {
				if(currentAccessedMember instanceof MethodCall) {
					MethodCall methodCall = (MethodCall)currentAccessedMember;
					methodCall.setReturnType(argument);
				}
			}
			else if(previousState.equals("Argument")) {
				if(currentAccessedMember instanceof MethodCall) {
					MethodCall methodCall = (MethodCall)currentAccessedMember;
					methodCall.addParameter(argument);
				}
			}
			
			previousState = "Argument";
		}*/
	}
	
	private void processChangePropagatedLine(String line) {
		if(currentOperationBodyDiff != null && !currentOperationBodyDiff.isEmpty()) {
			classDiff.addOperationBodyDiff(currentOperationBodyDiff);
		}
		if(line.contains("{Operation}")) {
			isChangePropagated = true;
			String signature = line.substring(line.lastIndexOf("{Operation}")+12, line.length());
			String methodName = signature.substring(0, signature.indexOf("("));
			boolean isConstructor = false;
			if(className.endsWith(methodName))
				isConstructor = true;
			String parameters = signature.substring(signature.indexOf("(")+1, signature.lastIndexOf(")"));
			String[] parameterTokens = parameters.split(",");
			StringBuilder sb = new StringBuilder();
			sb.append(methodName);
			String returnType = null;
			sb.append("(");
			for(int i=0; i<parameterTokens.length; i++) {
				String parameter = parameterTokens[i];
				if(i == 0 && !isConstructor) {
					returnType = parameter;
				}
				else {
					sb.append(parameter);
					if(i < parameterTokens.length-1)
						sb.append(", ");
				}
			}
			sb.append(")");
			if(returnType != null) {
				sb.append(" : ");
				sb.append(returnType);
			}
			String operation = sb.toString();
			
			UMLOperationBodyDiff operationBodyDiff = new UMLOperationBodyDiff(operation);
			currentOperationBodyDiff = operationBodyDiff;
		}
	}
	
	public UMLClassDiff getClassDiff() {
		return classDiff;
	}

	public static void main(String args[]) {
		File file = new File("usage1.xml.sct");
		String className = "org.jfree.chart.block.BlockContainer";
		//UMLModelDiff modelDiff = new UMLModelDiff();
//		EditScriptParser parser = new EditScriptParser(file, className);
//		parser.parse();
		//System.out.println(modelDiff);
	}
}
