package model.diff;
import generator.xml.XMLWriter;
import gr.uom.java.xmi.ASTReader;
import gr.uom.java.xmi.UMLClass;
import gr.uom.java.xmi.diff.UMLClassDiff;
import gr.uom.java.xmi.diff.UMLModelDiff;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.StringTokenizer;

import model.diff.parser.DiffParser;

public class SourceDiff extends ModelDiff{
	private List<UMLClass> addedClasses;
	private List<UMLClass> removedClasses;
	private DiffParser diffParser;
	private UMLModelDiff modelDiffVT;
	private boolean enableContainment;
	private boolean enableGeneralization;
	private boolean enableRealization;
	private boolean enableUsage;
	
	public SourceDiff(){
		this("");
	}

	public SourceDiff(String config_path){
		super(config_path);
		classesList = new ArrayList<String>();
		this.enableContainment = false;
		this.enableGeneralization = false;
		this.enableRealization = false;
		this.enableUsage = false;
	}
	
	public String diff(String className, String srcDirectory1, String srcDirectory2) throws  IOException {
		model1 = new ASTReader(new File(srcDirectory1)).getUmlModel();
		model2 = new ASTReader(new File(srcDirectory2)).getUmlModel();
		
		addedClasses = new ArrayList(model2.getSortedClassList());
		addedClasses.removeAll(model1.getSortedClassList());
		removedClasses = new ArrayList(model1.getSortedClassList());
		removedClasses.removeAll(model2.getSortedClassList());
		
		StringBuilder result = new StringBuilder();
		int classesSize;
		boolean isFullDiff = false;
		
		List<UMLClass> sortedClassList = null;
		if (className.equals("")){
			sortedClassList = model1.getSortedClassList();
			classesSize = sortedClassList.size();
			isFullDiff = true;
		}
		else{
			classesList = this.getClassesList(className);
			classesSize = classesList.size();
			isFullDiff = false;
		}

		for (int k = 0; k < classesSize; k++){ 
			modelDiffVT = new UMLModelDiff();
			
			String element;
			
			if (isFullDiff) element = sortedClassList.get(k).getName();
				else element = classesList.get(k);

			diffParser = new DiffParser(modelDiffVT, model1, model2,
					model1.getClass(element), model2.getClass(element), false);
			System.out.println("Class: <" + k + "> Doing the comparison for class "
					+ element + "...");

			if (element != null && model2.getClass(element) != null) {
				XMLWriter generator = new XMLWriter(model1, model2, element, enableContainment, 
						enableGeneralization, enableRealization, enableUsage);
				
				List<EditScriptResult> editScripts = null; 
				
				try {
					VTrackerStandalone vtracker = new VTrackerStandalone();
					editScripts = vtracker.run(generator.getXMLPairs());

				} catch (Exception e) {
					e.printStackTrace();
				}
				
				try {
					diffParser.parse(editScripts);
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
				
				String usage_result = "";
				byte[] usageEditScript = null;
				
				for(EditScriptResult esResult : editScripts )
					if (esResult.getType().equals("usage")) usageEditScript = esResult.getEditScript();
				if (usageEditScript != null){
					EditScriptParser parser = new EditScriptParser(usageEditScript, element);
					parser.parse();
					UMLClassDiff classDiff = parser.getClassDiff();
					
					if (!classDiff.toString().isEmpty()){
						usage_result = classDiff.toString();
					}
				}
				
				if (!modelDiffVT.toString().isEmpty()){
					result.append("\n" + element + ":");
					result.append("\n" + modelDiffVT.toString());
					result.append(usage_result);
				}else if (!usage_result.isEmpty()){
					result.append("\n" + element + ":");
					result.append(usage_result);
				}
			}
		}
		
		return result.toString();
	}
	
	/*
	 * Diff for containment, inheritance, and usage design aspects
	 */
	public void setFullDiff(){
		this.enableContainment = true;
		this.enableGeneralization = true;
		this.enableRealization = true;
		this.enableUsage = true;
	}
	
	public void setDiffConfiguration(boolean enableCont, boolean enableGeneralization, boolean enableRealization,
							boolean enableUsage){
		this.enableContainment = enableCont;
		this.enableGeneralization = enableGeneralization;
		this.enableRealization = enableRealization;
		this.enableUsage = enableUsage;
	}
	
	public List<UMLClass> getAddedClasses(){
		return this.addedClasses;
	}
	
	public List<UMLClass> getRemovedClasses(){
		return this.removedClasses;
	}
	
	public List<String> getClassList(){
		List<String> classList = new ArrayList<String>();
		List<UMLClass> sortedList = model1.getSortedClassList();
		for (int k = 0; k < sortedList.size(); k++)
			if (model2.getClass(sortedList.get(k).getName()) != null)
				classList.add(sortedList.get(k).getName());
		return classList;
	}
	
	private List<String> getClassesList(String classNames){
		List<String> classList = new ArrayList<String>();
		StringTokenizer st = new StringTokenizer(classNames, ";");
		
		while(st.hasMoreTokens())
			classList.add(st.nextToken().trim());
		
		return classList;
	}
}
