package model.diff.parser;

import java.io.BufferedReader;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.StringTokenizer;
import java.util.Map.Entry;

import model.diff.EditScriptResult;

import gr.uom.java.xmi.UMLAssociation;
import gr.uom.java.xmi.UMLAttribute;
import gr.uom.java.xmi.UMLClass;
import gr.uom.java.xmi.UMLClassifierRole;
import gr.uom.java.xmi.UMLCollaboration;
import gr.uom.java.xmi.UMLGeneralization;
import gr.uom.java.xmi.UMLMessage;
import gr.uom.java.xmi.UMLModel;
import gr.uom.java.xmi.UMLOperation;
import gr.uom.java.xmi.UMLParameter;
import gr.uom.java.xmi.UMLRealization;
import gr.uom.java.xmi.diff.UMLClassDiff;
import gr.uom.java.xmi.diff.UMLCollaborationDiff;
import gr.uom.java.xmi.diff.UMLModelDiff;

public class DiffParser {
	protected UMLModel model1;
	protected UMLModel model2;
	private UMLClass class1, class2;
	private UMLClassDiff umlClassDiff;
	private UMLModelDiff umlModelDiff;
	private List<Change> changes;
	private List<Change> genChanges;
	private List<Change> realChanges;
	private List<Change> usageChanges;
	private List<Change> collChanges;
	private List<Change> assocChanges;
	private List<Change> contChanges;
	protected List<Change> ucgenChanges;
	protected List<Change> ucassocChanges;
	protected List<Change> ucincludeChanges;
	protected List<Change> ucextendChanges;
	private boolean isXMIDiff;
	private boolean isUseCaseDiff;

	Map<String, UMLCollaborationDiff> collaborationDiffMap;

	public DiffParser(UMLModelDiff modelDiff, UMLModel model1, UMLModel model2, UMLClass class1, UMLClass class2,
			boolean isXMIDiff){

		this.isXMIDiff = isXMIDiff;
		this.changes = new ArrayList<Change>();
		this.genChanges = new ArrayList<Change>();
		this.realChanges = new ArrayList<Change>();
		this.usageChanges = new ArrayList<Change>();
		this.contChanges = new ArrayList<Change>();
		this.collChanges = new ArrayList<Change>();
		this.assocChanges = new ArrayList<Change>();
		this.ucassocChanges = new ArrayList<Change>();
		this.ucgenChanges = new ArrayList<Change>();
		this.ucincludeChanges = new ArrayList<Change>();
		this.ucextendChanges = new ArrayList<Change>();

		collaborationDiffMap = new HashMap<String, UMLCollaborationDiff>();

		this.umlModelDiff = modelDiff;
		this.model1 = model1;
		this.model2 = model2;
		this.class1 = class1;
		this.class2 = class2;
	}

	public void parse(List<EditScriptResult> editScripts) throws IOException{
		if (class1 != null) 
			umlClassDiff = new UMLClassDiff(class1.getName());
		String relationship; 
		
		for(EditScriptResult editScript : editScripts){			
			relationship = editScript.getType();
			if (relationship.equals("usage")) break;
			ByteArrayInputStream bstream = new ByteArrayInputStream(editScript.getEditScript());
			InputStreamReader istream = new InputStreamReader(bstream);
			BufferedReader reader = new BufferedReader(istream);
			
			String line;
			while((line = reader.readLine()) != null){
				StringTokenizer st = new StringTokenizer(line, "(,):=>");
				String changeType = null;
				if (st.hasMoreTokens()){
					changeType = st.nextToken();
				}

				String parent = null;
				if (st.hasMoreTokens()){
					parent = st.nextToken();
				}

				String child = null;
				if (st.hasMoreTokens()){
					child = st.nextToken();
				}
				if (changeType.equals("insert")){
					changes.add(new Change(1, parent, child,  relationship));
				}
				else if (changeType.equals("delete")){
					changes.add(new Change(2, parent, child,  relationship));
				}
				else if (changeType.equals("move")){
					changes.add(new Change(8, parent, child,  relationship));
				}
				else if (changeType.equals("change-propagated")){
					changes.add(new Change(16, parent, child,  relationship));
				}
				else if (changeType.equals("change")){
					Change change = new Change(4, parent, child,  relationship);
					List<String> attr = new ArrayList<String>();
					String changeOperation = null;
					String entity1 = null, entity2 =null, attribute = null;
					while(st.hasMoreTokens()){
						changeOperation = st.nextToken();
						if(st.hasMoreTokens()){
							attribute = st.nextToken();
						}
						if(st.hasMoreTokens()){
							if (st.hasMoreTokens())
								entity1 = st.nextToken();
						}
						if (changeOperation.equals("change") && st.hasMoreTokens()){
							if(st.hasMoreTokens()){
								if (st.hasMoreTokens())
									entity2 = st.nextToken();
							}
							if (attribute.equals("@className") || attribute.equals("@operationName")){
								attr.add("Rename: " + entity1 + " to " + entity2);
							}
							else attr.add("Change " + attribute + ": "  + entity1 + " to " + entity2);


						}
						else if (changeOperation.equals("delete")){
							attr.add("Deleted " + attribute + ": " + entity1);
						}
						else if (changeOperation.equals("insert")){
							attr.add("Inserted " + attribute + ": " + entity1);
						}

						change.setAttributes(attr);
						changes.add(change);
					}
				}
			}
		}
		splitChanges();
	}

	private void splitChanges(){
		for(int i = 0; i< changes.size(); i++){
			if (changes.get(i).getRelation().equals("gener"))
				genChanges.add(changes.get(i));
			else if (changes.get(i).getRelation().equals("realiz"))
				realChanges.add(changes.get(i));
			else if (changes.get(i).getRelation().equals("usage"))
				usageChanges.add(changes.get(i));
			else if (changes.get(i).getRelation().equals("cont"))
				contChanges.add(changes.get(i));
			else if (changes.get(i).getRelation().equals("collab"))
				collChanges.add(changes.get(i));
			else if (changes.get(i).getRelation().equals("assoc"))
				assocChanges.add(changes.get(i));
			else if (changes.get(i).getRelation().equals("ucassoc")) //use case association
				ucassocChanges.add(changes.get(i));
			else if (changes.get(i).getRelation().equals("ucgener")) //use case generalization
				ucgenChanges.add(changes.get(i));
			else if (changes.get(i).getRelation().equals("ucextend")) //use case extend
				ucextendChanges.add(changes.get(i));
			else if (changes.get(i).getRelation().equals("ucinclude")) //use case include
				ucincludeChanges.add(changes.get(i));
		}
		
		if (!isUseCaseDiff){
			if (isXMIDiff){
				this.setAssociationDiff();
				this.setCollaborationDiff();
			}
			this.setContainmentDiff();
			this.setGeneralizationDiff();
			this.setRealizationDiff();
		}
	}
	
	protected void setIsUSeCaseDiff(boolean b){
		isUseCaseDiff = b;
	}
	
	private void setContainmentDiff(){
		UMLAttribute attribute;
		UMLOperation operation;
		for(Change contChange : contChanges){
			if (contChange.getChangeType() == 1){
				if (contChange.getChild().startsWith("#")){
					attribute = findUMLAttribute(class2, contChange.getChild().substring(1));
					if (attribute != null)
						umlClassDiff.reportAddedAttribute(attribute);
					else{
						operation = findUMLOperation(class2, contChange.getChild().substring(1));
						if (operation != null){
							umlClassDiff.reportAddedOperation(operation);
						}
						else 
						{
							UMLOperation op = findUMLOperation(class2,contChange.getParent().substring(1));
							if (op != null){
								UMLParameter param = findUMLParameters(op,contChange.getChild().substring(1));
								if (param != null)
									umlClassDiff.reportChangedOperation(op, param.getName(), 1);

							}
						}
					}	
				}
				umlClassDiff.reportChangedOperation(findUMLOperation(class2,contChange.getParent().substring(1)),
						contChange.getChild(), 1);
			}
			if (contChange.getChangeType() == 2){
				if (contChange.getChild().startsWith("#")){
					attribute = findUMLAttribute(class1, contChange.getChild().substring(1));
					if (attribute != null){
						umlClassDiff.reportRemovedAttribute(attribute);
					}
					else{
						operation = findUMLOperation(class1, contChange.getChild().substring(1));
						if (operation != null)
							umlClassDiff.reportRemovedOperation(operation);
						else {
							UMLOperation op = null;
							if (!contChange.getParent().contains("VirtualRoot"))
								op = findUMLOperation(class1, contChange.getParent().substring(1));
							if (op != null){
								UMLParameter param = findUMLParameters(op, contChange.getChild().substring(1));
								if (param != null)
									umlClassDiff.reportChangedOperation(op, param.getName(), 2);
							}
						}
					}		
				}
				else if(!contChange.getParent().equals("VirtualRoot"))
					umlClassDiff.reportChangedOperation(findUMLOperation(class1,contChange.getParent().substring(1)),
							contChange.getChild(), 2);
			}
			if (contChange.getChangeType() == 4){
				if (contChange.getChild().startsWith("#")){
					UMLAttribute attribute1 = findUMLAttribute(class1, contChange.getParent().substring(1));
					UMLAttribute attribute2 = findUMLAttribute(class2, contChange.getChild().substring(1));
					if (attribute1 != null && attribute2 != null)
						umlClassDiff.reportChangedAttributes(attribute1, attribute2);
					else{
						UMLOperation operation1 = findUMLOperation(class1, contChange.getParent().substring(1));
						UMLOperation operation2 = findUMLOperation(class2, contChange.getChild().substring(1));
						if (operation1 != null && operation2 != null)
							umlClassDiff.reportChangedOperations(operation1, operation2);
					}		
				}
			}
			if (contChange.getChangeType() == 16){
				if (contChange.getChild().startsWith("#") && contChange.getParent().startsWith("#")){
					UMLOperation operation1 = findUMLOperation(class1, contChange.getParent().substring(1));
					UMLOperation operation2 = findUMLOperation(class2, contChange.getChild().substring(1));
					if (operation1 != null && operation2 != null)
						umlClassDiff.reportChangedOperations(operation1, operation2);
				}
			}
		}

		if (umlClassDiff != null)
			this.umlModelDiff.addUMLClassDiff(umlClassDiff);

	}

	private void setAssociationDiff(){
		for (Change change : assocChanges){
			if (change.getChangeType() == 1 && !change.getParent().contains("VirtualRoot") && !change.getParent().contains("#document")){
				UMLAssociation umlAssociation = findUMLAssociation(Long.parseLong(change.getParent().substring(1)),Long.parseLong(change.getChild().substring(1)), model2);
				if (umlAssociation != null)
					umlModelDiff.reportAddedAssociation(umlAssociation);
			}
			if (change.getChangeType() == 2 && !change.getParent().contains("VirtualRoot") && !change.getParent().contains("#document")){
				UMLAssociation umlAssociation = findUMLAssociation(Long.parseLong(change.getParent().substring(1)),Long.parseLong(change.getChild().substring(1)), model1);
				if (umlAssociation != null)
					umlModelDiff.reportRemovedAssociation(umlAssociation);
			}
		}
	}

	private void setCollaborationDiff(){
		UMLCollaborationDiff umlCollaborationDiff;
		for (Change change : collChanges){
			if (change.getParent().startsWith("#") && change.getChild().startsWith("#")){
				if (change.getChangeType() == 1){
					umlCollaborationDiff = getUMLCollaborationDiff(change.getParent().substring(1));
					UMLMessage umlMessage = findUMLMessage(Long.parseLong(change.getParent().substring(1)), 
							Long.parseLong(change.getChild().substring(1)), model2);
					if (umlMessage != null)
						umlCollaborationDiff.reportAddedMessage(umlMessage);
					else{
						UMLClassifierRole umlClassRole = findUMLClassifierRole(Long.parseLong(change.getParent().substring(1)), 
								Long.parseLong(change.getChild().substring(1)), model2);
						if (umlClassRole != null)
							umlCollaborationDiff.reportAddedClassifierRole(umlClassRole);
					}

				}
				else if (change.getChangeType() == 2){
					umlCollaborationDiff = getUMLCollaborationDiff(change.getParent().substring(1));
					UMLMessage umlMessage = findUMLMessage(Long.parseLong(change.getParent().substring(1)), 
							Long.parseLong(change.getChild().substring(1)), model1);
					if (umlMessage != null)
						umlCollaborationDiff.reportRemovedMessage(umlMessage);
					else{
						UMLClassifierRole umlClassRole = findUMLClassifierRole(Long.parseLong(change.getParent().substring(1)), 
								Long.parseLong(change.getChild().substring(1)), model1);
						if (umlClassRole != null)
							umlCollaborationDiff.reportAddedClassifierRole(umlClassRole);
					}
				}
			}
		}
		for(Entry<String, UMLCollaborationDiff> entry : this.collaborationDiffMap.entrySet())
			umlModelDiff.addUMLCollaborationDiff(entry.getValue());
	}
	
	private void setGeneralizationDiff(){
		for(Change genChange : genChanges){
			if(!genChange.getParent().equals("VirtualRoot") && !genChange.getParent().equals("#document")
					&& !genChange.getChild().equals("VirtualRoot")){ 
				if(genChange.getChangeType() == 1){ //added generalization
					umlModelDiff.reportAddedGeneralization(new UMLGeneralization(genChange.getParent(), 
							genChange.getChild()));
				}
				else if (genChange.getChangeType() == 2){ //removed generalization
					umlModelDiff.reportRemovedGeneralization(new UMLGeneralization(genChange.getParent(), 
							genChange.getChild()));
				}
				//else System.out.println("Oops: There is a change/move in generalization reported by VTracker");
			}
		}
	}

	private void setRealizationDiff(){
		for(Change realChange : realChanges){
			if(!realChange.getParent().contains("document") && !realChange.getParent().contains("VirtualRoot")
					&& !realChange.getChild().equals("VirtualRoot")){
				if (realChange.getChangeType() == 1){ //added generalization
					umlModelDiff.reportAddedRealization(new UMLRealization(realChange.getParent(), 
							realChange.getChild()));
				}
				else if (realChange.getChangeType() == 2){ //removed generalization);
					umlModelDiff.reportRemovedRealization(new UMLRealization(realChange.getParent(), 
							realChange.getChild()));
				}
				//else System.out.println("Oops: There is a change/move in realization reported by VTracker");
			}
		}
	}
	
	//--------------------------------------------------------------------------------------------------

	private UMLMessage findUMLMessage(long collaborationID, long messageID, UMLModel model){
		UMLCollaboration umlCollaboration = findUMLCollaboration(collaborationID, model);
		if (umlCollaboration != null){
			List<UMLMessage> messages = umlCollaboration.getMessageList();
			for (UMLMessage message : messages)
				if (message.getID() == messageID)
					return message;
		}
		return null;
	}

	private UMLClassifierRole findUMLClassifierRole(long collaborationID, long classRoleID, UMLModel model){
		UMLCollaboration umlCollaboration = findUMLCollaboration(collaborationID, model);
		if (umlCollaboration != null){
			List<UMLClassifierRole> classRolesList = umlCollaboration.getClassifierRoleList();
			for (UMLClassifierRole classRole : classRolesList)
				if (classRole.getID() == classRoleID)
					return classRole;
		}
		return null;
	}

	private UMLCollaborationDiff getUMLCollaborationDiff(String id){
		UMLCollaborationDiff umlCollaborationDiff = null;
		if (collaborationDiffMap.containsKey(id))
			umlCollaborationDiff = collaborationDiffMap.get(id);
		else {
			umlCollaborationDiff = new UMLCollaborationDiff(getCollaborationName(Long.parseLong(id), model2));
			collaborationDiffMap.put(id, umlCollaborationDiff);
		}
		return umlCollaborationDiff;
	}

	private UMLCollaboration findUMLCollaboration(long id, UMLModel model){
		List<UMLCollaboration> collaborationList =  model.getCollaborationList();
		for(UMLCollaboration umlCollaboration : collaborationList)
			if (umlCollaboration.getID() == id)
				return umlCollaboration;		
		return null;
	}

	private String getCollaborationName(long id, UMLModel model){
		List<UMLCollaboration> collaborationList =  model.getCollaborationList();
		for(UMLCollaboration umlCollaboration : collaborationList)
			if (umlCollaboration.getID() == id)
				return umlCollaboration.getName();		
		return null;
	}

	protected UMLAssociation findUMLAssociation(long assocID1, long assocID2, UMLModel umlModel){
		List<UMLAssociation> associationList = umlModel.getAssociationList();
		for(UMLAssociation umlAssociation : associationList)
			if (umlAssociation.getEnd1().getID() == assocID1 && umlAssociation.getEnd2().getID() == assocID2)
				return umlAssociation;
		return null;
	}
	
	private UMLAttribute findUMLAttribute(UMLClass classModel, String att){

		List<UMLAttribute> attrList = classModel.getAttributes();
		for(UMLAttribute attribute : attrList){
			if(attribute.getID() == Long.parseLong(att.trim())){
				return attribute;
			}
		}
		return null;
	}

	private UMLOperation findUMLOperation(UMLClass classModel, String oper){
		if (oper.contains("Root") || (oper.contains("document"))) return null;
		List<UMLOperation> operList = classModel.getOperations();
		for(UMLOperation operation : operList){
			if(operation.getID() == Long.parseLong(oper.trim())){
				return operation;
			}
		}
		return null;
	}


	private UMLParameter findUMLParameters(UMLOperation oper, String p){
		List<UMLParameter> operList = oper.getParameters();
		for(UMLParameter operation : operList){
			if(operation.getID() == Long.parseLong(p.trim())){
				return operation;
			}
			else return null;
		}
		return null;
	}


	public String getUMLModelDiffAsString(){
		return (umlModelDiff.toString());
	}

}
